# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields,osv

class product(osv.osv):
    _inherit = "product.product"
    _columns = {
        'track_all': fields.boolean('Track All Lots' , help="Forces to specify a Production Lot for all moves containing this product. This will also disable auto picking"),
    }
    def write(self, cr, uid, ids, values, context=None):
        if 'track_all' in values and values['track_all']:
            values.update({ 'track_production' : True, 'track_incoming' : True, 'track_outgoing' : True, 'auto_pick': False })
        return super(product,self).write(cr,uid,ids,values,context=context)
product()

class stock_move(osv.osv):
    _inherit = "stock.move"
    def _check_tracking_all(self, cr, uid, ids, context=None):
        """ Checks if production lot is assigned to stock move or not.
        @return: True or False
        """
        for move in self.browse(cr, uid, ids, context=context):
            if not move.prodlot_id and move.state == 'done' and move.product_id.track_all:
                return False
        return True
    _constraints = [
        (_check_tracking_all,
            'You must assign a production lot for this product',
            ['prodlot_id'])]
stock_move()

class stock_inventory_line(osv.osv):
    _inherit = "stock.inventory.line"
    _columns = {
        'product_track': fields.related('product_id','track_all',type='boolean'),
    }
    def on_change_product_id(self, cr, uid, ids, location_id, product, uom=False, to_date=False):
        res=super(stock_inventory_line,self).on_change_product_id(cr, uid, ids, location_id, product, uom=False, to_date=False)
        if not 'value' in res:
            res['value']={}
        if not product:
            res['value']['product_track']=False
            return res
        obj_product = self.pool.get('product.product').browse(cr, uid, product)
        res['value']['product_track']=obj_product.track_all
        return res

    def on_change_product_qty(self, cr, uid, ids, product_id, quantity, context=None):
        """
        Returns product tracking False in case of product quantity is Zero,
        which ultimately makes the production lot field as not mandatory.
        """
        track = self.pool.get('product.product').browse(cr, uid, product_id, context=context).track_all
        if not quantity:
            return {'value':{'product_track': False}}
        else:
            return {'value':{'product_track': track}}

stock_inventory_line()

class stock_partial_move_memory_out(osv.osv_memory):
    _inherit = "stock.move.memory.out"
    _columns = {
        'product_track': fields.related('product_id','track_all',type='boolean'),
    }

    def on_change_product_qty(self, cr, uid, ids, product_id, quantity, context=None):
        """
        Returns product tracking False in case of product quantity is Zero,
        which ultimately makes the production lot field as not mandatory.
        """
        track = self.pool.get('product.product').browse(cr, uid, product_id, context=context).track_all
        if not quantity:
            return {'value':{'product_track': False}}
        else:
            return {'value':{'product_track': track}}


stock_partial_move_memory_out()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
